'use strict';
/* global Backbone, _, $, AmCharts, notification */
/* global  mainview */
/* jshint browser: true , devel: true*/

(function($) {

  var GraphView;

  var sendAuthentication = function(xhr) {
  };

  /*
  __  __  ___  ___  ___ _    ___
 |  \/  |/ _ \|   \| __| |  / __|
 | |\/| | (_) | |) | _|| |__\__ \
 |_|  |_|\___/|___/|___|____|___/
   */

  var MainModel = Backbone.Model.extend({});

  var EventsModel = Backbone.Model.extend({
    initialize: function() {
      _.bindAll(this, 'processAdded');
      this.on('remove', function() {
        $('#output').empty();
      });

      this.on('add', function() {
        this.processAdded();
      });

    },
    getBranch: function(ts) {
      const y = ts.getFullYear().toString();
      const m = ts.getMonth().toString();
      const d = ts.getDate().toString();

      if (this.dateTree.hasOwnProperty(y)) {
        if (this.dateTree[y].hasOwnProperty(m)) {
          return this.dateTree[y][m][d];
        } else {
          return null;
        }
      } else {
        return null;
      }
    },
    findOccupancy: function(ts) {
      /*
        Get a branch from the date tree and see if the reduced set of records has a matching timestamp..
       */
      var count = 0;
      var tsDate = new Date(ts);
      var tsMS = tsDate.getTime();
      var branch = this.getBranch(tsDate);

      if (typeof branch === 'undefined' || branch === 'null') {
        return count;
      }
      _(branch).each(function(item) {
              if ((tsMS >= item.startMS) && (tsMS <= item.endMS)) {
                count = item.count;
                return count;
              }

            }, this);
      return count;
    },
    buildSpeedDateTree: function(occupancy) {
      /*
      Builds a tree to help speed up occupancy searching
       */
      var _tree = {};
      _(occupancy).each(function(item) {
              var newItem = item;
              var day, month,year;
              var _date = new Date(item.start);

              newItem.startMS = new Date(item.start).getTime();
              newItem.endMS = new Date(item.end).getTime();

              day = _date.getDate().toString();
              month = _date.getMonth().toString();
              year = _date.getFullYear().toString();

              if (!_tree.hasOwnProperty(year)) {
                _tree[year] = {};
              }

              if (_tree.hasOwnProperty(year)) {

                if (!_tree[year].hasOwnProperty(month)) {
                  _tree[year][month] = {};
                }

                if (_tree[year].hasOwnProperty(month)) {
                  if (!_tree[year][month].hasOwnProperty(day)) {
                    _tree[year][month][day] = [];
                  }

                  _tree[year][month][day].push(newItem);

                }

              }

            }, this);

      return _tree;
    },


    processAdded: function() {
      var skipOccupancy = false;
      console.time('processAdd');
      var tempCollection = new Backbone.Collection();

      var events;

      _.invoke(DeviceCollection.toArray(), 'destroy');

      events = this.get('events');

      this.dateTree = this.buildSpeedDateTree(events.occupancy);

      if (Object.keys(this.dateTree).length === 0) {
        skipOccupancy = true;
      }

      _(events.data).each(function(i) {
        var _occupancy = 0;

        if (!skipOccupancy) {
          _occupancy = this.findOccupancy(i.timestamp, events.occupancy);
        }

        tempCollection.add({
          dt: Date.create(i.timestamp).addHours(1),
          lux: i.lux,
          temp: i.temp,
          co2: i.co2,
          humid: i.humidity,
          noise: i.sound,
          occupancy: _occupancy
        });
      }, this);

      DeviceCollection.models = tempCollection.models;

      console.timeEnd('processAdd');
      DeviceCollection.trigger('update');

      notification.notify('success', 'Data loaded');

    }, decoder: function() {
      return {};
    }, dateTime: function($date) {
      var dateTime = new Date.create($date);
      var date = dateTime.format('{yyyy}-{MM}-{dd}');
      var time = dateTime.format('{HH}:{mm}:{ss}');
      return {
        dateTime: dateTime.format('{yyyy}-{MM}-{dd} {HH}:{mm}:{ss}'),
        date: date,
        time: time
      };
    }
  });

  /*
   ___ ___  _    _    ___ ___ _____ ___ ___  _  _ ___
  / __/ _ \| |  | |  | __/ __|_   _|_ _/ _ \| \| / __|
 | (_| (_) | |__| |__| _| (__  | |  | | (_) | .` \__ \
  \___\___/|____|____|___\___| |_| |___\___/|_|\_|___/

   */

  var mDotCollection = Backbone.Collection.extend({
    model: EventsModel,
    url: 'https://qz0da4.internetofthings.ibmcloud.com/api/v0002/historian/types/mDot/devices/',
    initialize: function() {
      this.on('update', function() {
        // Nothing
      });
    }

  });

  /*
 __   _____ _____      _____
 \ \ / /_ _| __\ \    / / __|
  \ V / | || _| \ \/\/ /\__ \
   \_/ |___|___| \_/\_/ |___/
   */

  var MDOT = Backbone.View.extend({
    model: EventsModel, el: $('#output'),

    events: {
      'click button#refresh': 'refresh'
    }, initialize: function() {
      _.bindAll(this, 'render', 'refresh', 'update');
      this.collection.bind('change reset add remove', this.render, this);

      this.template = _.template($('#loaded-template').html());
    }, refresh: function() {

    }, update: function() {
      this.collection.each(function() {
      });

    }, render: function() {
      $('#output').empty();
      return this;
    }
  });



  var MainView = Backbone.View.extend({
    el: $('#main'), template: _.template($('#main-template').html()), events: {
      'change select#device': 'changeDevice',
      'click button#refresh': 'updateDevice',
      submit: function() {
        // Catch the submit
      }

    }, initialize: function() {
      _.bindAll(this, 'render', 'changeDevice', 'updateDevice');
      this.model.on('change', this.updateDevice);
      this.render();
    }, render: function() {
      $(this.el).html(this.template());
      return this;
    }, changeDevice: function() {
      var newDevice;
      newDevice = this.$el.find('#device')[0].value;
      this.model.set('device', newDevice);
    }, updateDevice: function() {
      var fetchObj = {beforeSend: sendAuthentication};
      notification.clearAll();
      if (this.model.has('device')) {
        this.collection.url = '/apiv2/mdot/' + this.model.get('device');
        $('#output').empty();
        notification.notify('info', 'Loading...');
        this.collection.fetch(fetchObj);
      } else {
        console.error('Nothing to get!');
      }

    }

  });

  GraphView = Backbone.View.extend({
    el: $('#graph'), initialize: function() {
      this.modes = ['', 'lux', 'temp', 'co2', 'humid', 'noise'];
      this.titles = [
        '', 'Light Levels', 'Temperature', 'Co2 Levels', 'Humidity', 'Sound'
      ];

      this.mode = 0;

      // Config AMChart
      this.chart = {};
      this.categoryAxesSettings = new AmCharts.CategoryAxesSettings();
      this.dataSet = new AmCharts.DataSet();

      _.bindAll(this, 'render', 'updateGraphV2', 'setupChart');
      this.collection.on('update', function() {
        // Trigger the redraw
        this.updateGraphV2();
      }, this);

    }, events: {
      'change select#displaymode': 'changeMode'
    }, setupChart: function() {
      this.categoryAxesSettings.minPeriod = 'mm';
      this.chart.categoryAxesSettings = this.categoryAxesSettings;

      this.dataSet.color = '#b0de09';

      this.chart.dataSets = [this.dataSet];
    },

    doChartV2: function(chartData) {
      var self = this;
      console.time('doChartV2');
      self.chart = AmCharts.makeChart('chartdiv', {
        type: 'serial',
        theme: 'light',

        legend: {
          useGraphSettings: true,
          color: '#fff',
          switchColor: '#556374'
        },
        color: '#ffffff',
        dataProvider: chartData,
        dataDateFormat: 'YYYY-MM-DDTHH:NN:SS.QQQ',
        synchronizeGrid: true,
        valueAxes: [
          {
            id: 'lux',
            axisColor: '#FFC802',
            axisThickness: 2,
            axisAlpha: 1,
            position: 'left',
            gridColor: '#556374'
          },
          {
            id: 'co2',
            axisColor: 'rgba(0,191,255,1)',
            axisThickness: 2,
            axisAlpha: 1,
            position: 'right',
            gridColor: '#556374'
          },
          {
            id: 'temp',
            axisColor: 'rgba(46,255,0,1)',
            axisThickness: 2,
            gridAlpha: 0,
            offset: 50,
            axisAlpha: 1,
            position: 'left',
            gridColor: '#556374'
          },
          {
            id: 'humid',
            axisColor: 'rgba(255,0,99,1)',
            axisThickness: 2,
            axisAlpha: 1,
            offset: 50,
            position: 'right',
            gridColor: '#556374'
          },
          {
                      id: 'occupancy',
                      axisColor: '#aaaaaa',
                      axisThickness: 2,
                      gridAlpha: 0,
                      offset: 100,
                      axisAlpha: 1,
                      position: 'right',
                      gridColor: '#556374'
                    }
        ],
        graphs: [{id: 'occ',
                              valueAxis: 'occupancy',
                              type: 'line',
                              clustered: false,
                              columnWidth: 1,
                              lineColor: '#aaaaaa',
                              title: 'Occupancy',
                              valueField: 'occupancy',
                              fillColor: '#888888',
                              fillAlphas: 0.2,
                              fillToAxis: 'x'
                            },
          {
            valueAxis: 'lux',
            lineColor: '#FFC802',
            title: 'Light Level',
            valueField: 'lux',
            fillAlphas: 0
          },
          {
            valueAxis: 'co2',
            lineColor: 'rgba(0,191,255,1)',
            title: 'Co2',
            valueField: 'co2',
            fillAlphas: 0
          },
          {
            valueAxis: 'temp',
            lineColor: 'rgba(46,255,0,1)',
            title: 'Temperature',
            valueField: 'temp',
            fillAlphas: 0
          },
          {
            valueAxis: 'humid',
            lineColor: 'rgba(255,0,99,1)',
            title: 'Humidity',
            valueField: 'humid',
            fillAlphas: 0
          }
        ],
        chartScrollbar: {
          graph: 'occ',oppositeAxis: false,
          offset: 30,
          scrollbarHeight: 80,
          backgroundAlpha: 0,
          selectedBackgroundAlpha: 0.1,
          selectedBackgroundColor: '#888888',
          graphFillAlpha: 0,
          graphLineAlpha: 0.5,
          selectedGraphFillAlpha: 0,
          selectedGraphLineAlpha: 1,
          autoGridCount: true,
          color: '#AAAAAA',
          updateOnReleaseOnly: true
        },
        chartCursor: {
          cursorPosition: 'mouse',
          cursorColor: '#14bfff',
          color: '#000'
        },
        categoryField: 'date',
        categoryAxis: {
          minPeriod: '15mm',
          parseDates: true,
          axisColor: 'rgba(255,255,255,0.8)',
          minorGridEnabled: true,
          gridColor: '#556374'
        },
        export: {
          enabled: true, position: 'bottom-right'
        }
      });

      $('#chartdiv').empty();
      self.chart.write('chartdiv');

      console.timeEnd('doChartV2');
    }, updateGraphV2: function() {
      console.time('updateGraphV2');
      var self = this;
      var chartData = [];
      console.time('chartData');
      _(this.collection.models).each(function(i) {
        chartData.push({
          date: i.get('dt'),
          co2: i.get('co2'),
          humid: i.get('humid'),
          lux: i.get('lux'),
          noise: i.get('noise'),
          temp: i.get('temp'),
          occupancy: i.get('occupancy')
        });
      });
      console.timeEnd('chartData');

      self.doChartV2(chartData);
      console.timeEnd('updateGraphV2');
    }

  });

  notification.configProfile('global', {
    stacking: false
  });
  var DeviceCollection = new Backbone.Collection;
  var mdotCollection = new mDotCollection();

  var mainSettings = new MainModel();
  var views = {};
  views.mainview = new MainView({
    collection: mdotCollection, model: mainSettings
  });

  views.mdot = new MDOT({collection: mdotCollection});

  views.grapher = new GraphView({collection: DeviceCollection});

})(jQuery);
